@@ -16,12 +16,14 @@
  */
 package org.apache.spark.api.python
 
+import org.apache.gluten.columnarbatch.ArrowBatches.ArrowJavaBatch
 import org.apache.gluten.columnarbatch.ColumnarBatches
 import org.apache.gluten.exception.GlutenException
-import org.apache.gluten.extension.GlutenPlan
+import org.apache.gluten.execution.GlutenPlan
+import org.apache.gluten.extension.columnar.transition.{Convention, ConventionReq}
+import org.apache.gluten.iterator.Iterators
 import org.apache.gluten.memory.arrow.alloc.ArrowBufferAllocators
 import org.apache.gluten.utils.PullOutProjectHelper
-import org.apache.gluten.utils.iterator.Iterators
 import org.apache.gluten.vectorized.ArrowWritableColumnVector
 
 import org.apache.spark.{ContextAwareIterator, SparkEnv, TaskContext}
@@ -210,7 +212,13 @@ case class ColumnarArrowEvalPythonExec(
     evalType: Int)
   extends EvalPythonExec
   with GlutenPlan {
-  override def supportsColumnar: Boolean = true
+
+  override def batchType(): Convention.BatchType = ArrowJavaBatch
+
+  override def rowType0(): Convention.RowType = Convention.RowType.None
+
+  override def requiredChildConvention(): Seq[ConventionReq] = List(
+    ConventionReq.ofBatch(ConventionReq.BatchType.Is(ArrowJavaBatch)))
 
   override lazy val metrics = Map(
     "numOutputRows" -> SQLMetrics.createMetric(sparkContext, "number of output rows"),
@@ -334,7 +342,7 @@ case class ColumnarArrowEvalPythonExec(
         val inputBatchIter = contextAwareIterator.map {
           inputCb =>
             start_time = System.nanoTime()
-            ColumnarBatches.ensureLoaded(ArrowBufferAllocators.contextInstance, inputCb)
+            ColumnarBatches.checkLoaded(inputCb)
             ColumnarBatches.retain(inputCb)
             // 0. cache input for later merge
             inputCbCache += inputCb
@@ -366,11 +374,9 @@ case class ColumnarArrowEvalPythonExec(
               numOutputBatches += 1
               numOutputRows += numRows
               val batch = new ColumnarBatch(joinedVectors, numRows)
-              val offloaded =
-                ColumnarBatches.ensureOffloaded(ArrowBufferAllocators.contextInstance, batch)
-              ColumnarBatches.release(outputCb)
+              ColumnarBatches.checkLoaded(batch)
               procTime += (System.nanoTime() - start_time) / 1000000
-              offloaded
+              batch
           }
         Iterators
           .wrap(res)
@@ -390,13 +396,13 @@ case class ColumnarArrowEvalPythonExec(
     if (from > to) {
       do {
         vector.close()
-      } while (vector.refCnt() == to)
+      } while (vector.refCnt() != to)
       return
     }
     // from < to
     do {
       vector.retain()
-    } while (vector.refCnt() == to)
+    } while (vector.refCnt() != to)
   }
 
   override protected def withNewChildInternal(newChild: SparkPlan): ColumnarArrowEvalPythonExec =
