@@ -16,20 +16,21 @@
  */
 package org.apache.gluten.execution
 
-import org.apache.gluten.GlutenConfig
+import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.extension.columnar.validator.FallbackInjects
 
 import org.apache.spark.SparkConf
 import org.apache.spark.sql.catalyst.expressions.aggregate.{Final, Partial}
 import org.apache.spark.sql.execution.aggregate.BaseAggregateExec
-import org.apache.spark.sql.functions._
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.types._
 
+import java.sql.Timestamp
+
 abstract class VeloxAggregateFunctionsSuite extends VeloxWholeStageTransformerSuite {
 
   protected val rootPath: String = getClass.getResource("/").getPath
-  override protected val resourcePath: String = "/tpch-data-parquet-velox"
+  override protected val resourcePath: String = "/tpch-data-parquet"
   override protected val fileFormat: String = "parquet"
 
   import testImplicits._
@@ -48,6 +49,7 @@ abstract class VeloxAggregateFunctionsSuite extends VeloxWholeStageTransformerSu
       .set("spark.unsafe.exceptionOnMemoryLeak", "true")
       .set("spark.sql.autoBroadcastJoinThreshold", "-1")
       .set("spark.sql.sources.useV1SourceList", "avro")
+      .set("spark.gluten.sql.mergeTwoPhasesAggregate.enabled", "false")
   }
 
   test("count") {
@@ -565,6 +567,18 @@ abstract class VeloxAggregateFunctionsSuite extends VeloxWholeStageTransformerSu
       "select approx_count_distinct(l_discount), count(distinct l_orderkey) from lineitem") {
       checkGlutenOperatorMatch[HashAggregateExecTransformer]
     }
+    withTempPath {
+      path =>
+        val t1 = Timestamp.valueOf("2024-08-22 10:10:10.010")
+        val t2 = Timestamp.valueOf("2014-12-31 00:00:00.012")
+        val t3 = Timestamp.valueOf("1968-12-31 23:59:59.001")
+        Seq(t1, t2, t3).toDF("t").write.parquet(path.getCanonicalPath)
+
+        spark.read.parquet(path.getCanonicalPath).createOrReplaceTempView("view")
+        runQueryAndCompare("select approx_count_distinct(t) from view") {
+          checkGlutenOperatorMatch[HashAggregateExecTransformer]
+        }
+    }
   }
 
   test("max_by") {
@@ -1127,13 +1141,10 @@ abstract class VeloxAggregateFunctionsSuite extends VeloxWholeStageTransformerSu
               StructField("lastUpdated", LongType, true),
               StructField("version", LongType, true))),
           true)))
-    val df = spark.read.schema(jsonSchema).json(Seq(jsonStr).toDS)
-    df.select(collect_set(col("txn"))).collect
-
-    df.select(min(col("txn"))).collect
-
-    df.select(max(col("txn"))).collect
-
+    spark.read.schema(jsonSchema).json(Seq(jsonStr).toDS).createOrReplaceTempView("t1")
+    runQueryAndCompare("select collect_set(txn), min(txn), max(txn) from t1") {
+      checkGlutenOperatorMatch[HashAggregateExecTransformer]
+    }
   }
 
   test("drop redundant partial sort which has pre-project when offload sortAgg") {
@@ -1170,7 +1181,7 @@ class VeloxAggregateFunctionsDefaultSuite extends VeloxAggregateFunctionsSuite {
       .set(GlutenConfig.VELOX_FLUSHABLE_PARTIAL_AGGREGATION_ENABLED.key, "false")
   }
 
-  test("group sets with keys") {
+  test("flushable aggregate rule") {
     withSQLConf(SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> "false") {
       runQueryAndCompare(VeloxAggregateFunctionsSuite.GROUP_SETS_TEST_SQL) {
         df =>
@@ -1208,6 +1219,24 @@ class VeloxAggregateFunctionsFlushSuite extends VeloxAggregateFunctionsSuite {
     }
   }
 
+  test("flushable aggregate rule - agg input already distributed by keys") {
+    withSQLConf(
+      SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> "false",
+      SQLConf.FILES_MAX_PARTITION_BYTES.key -> "1k") {
+      runQueryAndCompare(
+        "select * from (select distinct l_orderkey,l_partkey from lineitem) a" +
+          " inner join (select l_orderkey from lineitem limit 10) b" +
+          " on a.l_orderkey = b.l_orderkey limit 10") {
+        df =>
+          val executedPlan = getExecutedPlan(df)
+          assert(
+            executedPlan.exists(plan => plan.isInstanceOf[RegularHashAggregateExecTransformer]))
+          assert(
+            executedPlan.exists(plan => plan.isInstanceOf[FlushableHashAggregateExecTransformer]))
+      }
+    }
+  }
+
   test("flushable aggregate decimal sum") {
     withSQLConf(
       SQLConf.ADAPTIVE_EXECUTION_ENABLED.key -> "false",
