@@ -17,28 +17,26 @@
 #include "CHColumnToSparkRow.h"
 #include <Columns/ColumnArray.h>
 #include <Columns/ColumnConst.h>
-#include <Columns/ColumnDecimal.h>
 #include <Columns/ColumnMap.h>
 #include <Columns/ColumnNullable.h>
 #include <Columns/ColumnString.h>
 #include <Columns/IColumn.h>
-#include <Core/Types.h>
 #include <DataTypes/DataTypeArray.h>
 #include <DataTypes/DataTypeLowCardinality.h>
 #include <DataTypes/DataTypeMap.h>
 #include <DataTypes/DataTypeNullable.h>
-#include <DataTypes/DataTypeString.h>
 #include <DataTypes/DataTypeTuple.h>
 #include <DataTypes/DataTypesDecimal.h>
 #include <DataTypes/ObjectUtils.h>
+#include <jni/jni_common.h>
 #include <Common/Exception.h>
 
 namespace DB
 {
 namespace ErrorCodes
 {
-    extern const int LOGICAL_ERROR;
-    extern const int UNKNOWN_TYPE;
+extern const int LOGICAL_ERROR;
+extern const int UNKNOWN_TYPE;
 }
 }
 
@@ -297,11 +295,7 @@ static void writeValue(
 }
 
 SparkRowInfo::SparkRowInfo(
-    const DB::ColumnsWithTypeAndName & cols,
-    const DB::DataTypes & dataTypes,
-    const size_t & col_size,
-    const size_t & row_size,
-    const MaskVector & masks)
+    const DB::ColumnsWithTypeAndName & cols, const DB::DataTypes & dataTypes, size_t col_size, size_t row_size, const MaskVector & masks)
     : types(dataTypes)
     , num_rows(masks == nullptr ? row_size : masks->size())
     , num_cols(col_size)
@@ -461,8 +455,7 @@ std::unique_ptr<SparkRowInfo> CHColumnToSparkRow::convertCHColumnToSparkRow(cons
         const auto & col = block.getByPosition(col_idx);
         int64_t field_offset = spark_row_info->getFieldOffset(col_idx);
 
-        ColumnWithTypeAndName col_full{col.column->convertToFullIfNeeded(),
-            removeLowCardinality(col.type), col.name};
+        ColumnWithTypeAndName col_full{col.column->convertToFullIfNeeded(), removeLowCardinality(col.type), col.name};
         writeValue(
             spark_row_info->getBufferAddress(),
             field_offset,
@@ -501,7 +494,7 @@ int64_t BackingDataLengthCalculator::calculate(const Field & field) const
 
     if (which.isStringOrFixedString())
     {
-        const auto & str = field.get<String>();
+        const auto & str = field.safeGet<String>();
         return roundNumberOfBytesToNearestWord(str.size());
     }
 
@@ -511,7 +504,7 @@ int64_t BackingDataLengthCalculator::calculate(const Field & field) const
     if (which.isArray())
     {
         /// 内存布局：numElements(8B) | null_bitmap(与numElements成正比) | values(每个值长度与类型有关) | backing buffer
-        const auto & array = field.get<Array>(); /// Array can not be wrapped with Nullable
+        const auto & array = field.safeGet<Array>(); /// Array can not be wrapped with Nullable
         const auto num_elems = array.size();
         int64_t res = 8 + calculateBitSetWidthInBytes(num_elems);
 
@@ -531,7 +524,7 @@ int64_t BackingDataLengthCalculator::calculate(const Field & field) const
         int64_t res = 8;
 
         /// Construct Array of keys and values from Map
-        const auto & map = field.get<Map>(); /// Map can not be wrapped with Nullable
+        const auto & map = field.safeGet<Map>(); /// Map can not be wrapped with Nullable
         const auto num_keys = map.size();
         auto array_key = Array();
         auto array_val = Array();
@@ -539,7 +532,7 @@ int64_t BackingDataLengthCalculator::calculate(const Field & field) const
         array_val.reserve(num_keys);
         for (size_t i = 0; i < num_keys; ++i)
         {
-            const auto & pair = map[i].get<DB::Tuple>();
+            const auto & pair = map[i].safeGet<DB::Tuple>();
             array_key.push_back(pair[0]);
             array_val.push_back(pair[1]);
         }
@@ -561,7 +554,7 @@ int64_t BackingDataLengthCalculator::calculate(const Field & field) const
     if (which.isTuple())
     {
         /// 内存布局：null_bitmap(字节数与字段数成正比) | field1 value(8B) | field2 value(8B) | ... | fieldn value(8B) | backing buffer
-        const auto & tuple = field.get<Tuple>(); /// Tuple can not be wrapped with Nullable
+        const auto & tuple = field.safeGet<Tuple>(); /// Tuple can not be wrapped with Nullable
         const auto * type_tuple = typeid_cast<const DataTypeTuple *>(type_without_nullable.get());
         const auto & type_fields = type_tuple->getElements();
         const auto num_fields = type_fields.size();
@@ -585,13 +578,11 @@ int64_t BackingDataLengthCalculator::getArrayElementSize(const DataTypePtr & nes
         return 1;
     else if (nested_which.isUInt16() || nested_which.isInt16() || nested_which.isDate())
         return 2;
-    else if (
-        nested_which.isUInt32() || nested_which.isInt32() || nested_which.isFloat32() || nested_which.isDate32()
-        || nested_which.isDecimal32())
+    else if (nested_which.isUInt32() || nested_which.isInt32() || nested_which.isFloat32() || nested_which.isDate32())
         return 4;
     else if (
         nested_which.isUInt64() || nested_which.isInt64() || nested_which.isFloat64() || nested_which.isDateTime64()
-        || nested_which.isDecimal64())
+        || nested_which.isDecimal32() || nested_which.isDecimal64())
         return 8;
     else
         return 8;
@@ -687,26 +678,7 @@ int64_t VariableLengthDataWriter::writeArray(size_t row_idx, const DB::Array & a
             if (elem.isNull())
                 bitSet(buffer_address + offset + start + 8, i);
             else
-            {
-                if (writer.getWhichDataType().isFloat32())
-                {
-                    // We can not use get<char>() directly here to process Float32 field,
-                    // because it will get 8 byte data, but Float32 is 4 byte, which will cause error conversion.
-                    auto v = static_cast<Float32>(elem.get<Float32>());
-                    writer.unsafeWrite(
-                        reinterpret_cast<const char *>(&v), buffer_address + offset + start + 8 + len_null_bitmap + i * elem_size);
-                }
-                else if (writer.getWhichDataType().isFloat64())
-                {
-                    // Fix 'Invalid Field get from type Float64 to type Int64' in debug build.
-                    auto v = elem.get<Float64>();
-                    writer.unsafeWrite(reinterpret_cast<const char *>(&v), buffer_address + offset + start + 8 + len_null_bitmap + i * elem_size);
-                }
-                else
-                    writer.unsafeWrite(
-                        reinterpret_cast<const char *>(&elem.get<char>()),
-                        buffer_address + offset + start + 8 + len_null_bitmap + i * elem_size);
-            }
+                writer.write(elem, buffer_address + offset + start + 8 + len_null_bitmap + i * elem_size);
         }
     }
     else
@@ -749,7 +721,7 @@ int64_t VariableLengthDataWriter::writeMap(size_t row_idx, const DB::Map & map,
     val_array.reserve(num_pairs);
     for (size_t i = 0; i < num_pairs; ++i)
     {
-        const auto & pair = map[i].get<DB::Tuple>();
+        const auto & pair = map[i].safeGet<DB::Tuple>();
         key_array.push_back(pair[0]);
         val_array.push_back(pair[1]);
     }
@@ -807,27 +779,7 @@ int64_t VariableLengthDataWriter::writeStruct(size_t row_idx, const DB::Tuple &
         if (BackingDataLengthCalculator::isFixedLengthDataType(removeNullable(field_type)))
         {
             FixedLengthDataWriter writer(field_type);
-            if (writer.getWhichDataType().isFloat32())
-            {
-                // We can not use get<char>() directly here to process Float32 field,
-                // because it will get 8 byte data, but Float32 is 4 byte, which will cause error conversion.
-                auto v = static_cast<Float32>(field_value.get<Float32>());
-                writer.unsafeWrite(reinterpret_cast<const char *>(&v), buffer_address + offset + start + len_null_bitmap + i * 8);
-            }
-            else if (writer.getWhichDataType().isFloat64())
-            {
-                // Fix 'Invalid Field get from type Float64 to type Int64' in debug build.
-                auto v = field_value.get<Float64>();
-                writer.unsafeWrite(reinterpret_cast<const char *>(&v), buffer_address + offset + start + len_null_bitmap + i * 8);
-            }
-            else if (writer.getWhichDataType().isDecimal64() || writer.getWhichDataType().isDateTime64())
-            {
-                auto v = field_value.get<Decimal64>();
-                writer.unsafeWrite(reinterpret_cast<const char *>(&v), buffer_address + offset + start + len_null_bitmap + i * 8);
-            }
-            else
-                writer.unsafeWrite(
-                    reinterpret_cast<const char *>(&field_value.get<char>()), buffer_address + offset + start + len_null_bitmap + i * 8);
+            writer.write(field_value, buffer_address + offset + start + len_null_bitmap + i * 8);
         }
         else
         {
@@ -848,7 +800,7 @@ int64_t VariableLengthDataWriter::write(size_t row_idx, const DB::Field & field,
 
     if (which.isStringOrFixedString())
     {
-        const auto & str = field.get<String>();
+        const auto & str = field.safeGet<String>();
         return writeUnalignedBytes(row_idx, str.data(), str.size(), parent_offset);
     }
 
@@ -863,19 +815,19 @@ int64_t VariableLengthDataWriter::write(size_t row_idx, const DB::Field & field,
 
     if (which.isArray())
     {
-        const auto & array = field.get<Array>();
+        const auto & array = field.safeGet<Array>();
         return writeArray(row_idx, array, parent_offset);
     }
 
     if (which.isMap())
     {
-        const auto & map = field.get<Map>();
+        const auto & map = field.safeGet<Map>();
         return writeMap(row_idx, map, parent_offset);
     }
 
     if (which.isTuple())
     {
-        const auto & tuple = field.get<Tuple>();
+        const auto & tuple = field.safeGet<Tuple>();
         return writeStruct(row_idx, tuple, parent_offset);
     }
 
@@ -921,64 +873,64 @@ void FixedLengthDataWriter::write(const DB::Field & field, char * buffer)
 
     if (which.isUInt8())
     {
-        const auto value = UInt8(field.get<UInt8>());
+        const auto value = static_cast<UInt8>(field.safeGet<UInt8>());
         memcpy(buffer, &value, 1);
     }
     else if (which.isUInt16() || which.isDate())
     {
-        const auto value = UInt16(field.get<UInt16>());
+        const auto value = static_cast<UInt16>(field.safeGet<UInt16>());
         memcpy(buffer, &value, 2);
     }
     else if (which.isUInt32() || which.isDate32())
     {
-        const auto value = UInt32(field.get<UInt32>());
+        const auto value = static_cast<UInt32>(field.safeGet<UInt32>());
         memcpy(buffer, &value, 4);
     }
     else if (which.isUInt64())
     {
-        const auto & value = field.get<UInt64>();
+        const auto & value = field.safeGet<UInt64>();
         memcpy(buffer, &value, 8);
     }
     else if (which.isInt8())
     {
-        const auto value = Int8(field.get<Int8>());
+        const auto value = static_cast<Int8>(field.safeGet<Int8>());
         memcpy(buffer, &value, 1);
     }
     else if (which.isInt16())
     {
-        const auto value = Int16(field.get<Int16>());
+        const auto value = static_cast<Int16>(field.safeGet<Int16>());
         memcpy(buffer, &value, 2);
     }
     else if (which.isInt32())
     {
-        const auto value = Int32(field.get<Int32>());
+        const auto value = static_cast<Int32>(field.safeGet<Int32>());
         memcpy(buffer, &value, 4);
     }
     else if (which.isInt64())
     {
-        const auto & value = field.get<Int64>();
+        const auto & value = field.safeGet<Int64>();
         memcpy(buffer, &value, 8);
     }
     else if (which.isFloat32())
     {
-        const auto value = Float32(field.get<Float32>());
+        const auto value = static_cast<Float32>(field.safeGet<Float32>());
         memcpy(buffer, &value, 4);
     }
     else if (which.isFloat64())
     {
-        const auto & value = field.get<Float64>();
+        const auto & value = field.safeGet<Float64>();
         memcpy(buffer, &value, 8);
     }
     else if (which.isDecimal32())
     {
-        const auto & value = field.get<Decimal32>();
+        const auto & value = field.safeGet<Decimal32>();
         const Int64 decimal = static_cast<Int64>(value.getValue());
         memcpy(buffer, &decimal, 8);
     }
     else if (which.isDecimal64() || which.isDateTime64())
     {
-        const auto & value = field.get<Decimal64>();
-        auto decimal = value.getValue();
+        const auto & value = field.safeGet<Decimal64>();
+        const auto decimal = value.getValue();
         memcpy(buffer, &decimal, 8);
     }
     else
@@ -995,4 +947,34 @@ void FixedLengthDataWriter::unsafeWrite(const char * __restrict src, char * __re
     memcpy(buffer, src, type_without_nullable->getSizeOfValueInMemory());
 }
 
+namespace SparkRowInfoJNI
+{
+static jclass spark_row_info_class;
+static jmethodID spark_row_info_constructor;
+void init(JNIEnv * env)
+{
+    spark_row_info_class = CreateGlobalClassReference(env, "Lorg/apache/gluten/row/SparkRowInfo;");
+    spark_row_info_constructor = GetMethodID(env, spark_row_info_class, "<init>", "([J[JJJJ)V");
+}
+void destroy(JNIEnv * env)
+{
+    env->DeleteGlobalRef(spark_row_info_class);
+}
+jobject create(JNIEnv * env, const SparkRowInfo & spark_row_info)
+{
+    auto * offsets_arr = env->NewLongArray(spark_row_info.getNumRows());
+    const auto * offsets_src = spark_row_info.getOffsets().data();
+    env->SetLongArrayRegion(offsets_arr, 0, spark_row_info.getNumRows(), static_cast<const jlong *>(offsets_src));
+    auto * lengths_arr = env->NewLongArray(spark_row_info.getNumRows());
+    const auto * lengths_src = spark_row_info.getLengths().data();
+    env->SetLongArrayRegion(lengths_arr, 0, spark_row_info.getNumRows(), static_cast<const jlong *>(lengths_src));
+    int64_t address = reinterpret_cast<int64_t>(spark_row_info.getBufferAddress());
+    int64_t column_number = spark_row_info.getNumCols();
+    int64_t total_size = spark_row_info.getTotalBytes();
+
+    return env->NewObject(
+        spark_row_info_class, spark_row_info_constructor, offsets_arr, lengths_arr, address, column_number, total_size);
+}
+}
+
 }
