@@ -20,7 +20,6 @@
 
 #if USE_PARQUET
 
-#include <memory>
 #include <IO/WriteBuffer.h>
 #include <Storages/Output/OutputFormatFile.h>
 
@@ -32,9 +31,8 @@ public:
     explicit ParquetOutputFormatFile(
         DB::ContextPtr context_,
         const std::string & file_uri_,
-        WriteBufferBuilderPtr write_buffer_builder_,
-        const std::vector<std::string> & preferred_column_names_);
-    ~ParquetOutputFormatFile() override = default;
+        const WriteBufferBuilderPtr & write_buffer_builder_,
+        const DB::Block & preferred_schema_);
 
     OutputFormatFile::OutputFormatPtr createOutputFormat(const DB::Block & header) override;
 };
