@@ -23,32 +23,28 @@
 #include <Interpreters/Context.h>
 #include <Interpreters/HashJoin/HashJoin.h>
 #include <Interpreters/TableJoin.h>
-#include <Interpreters/TreeRewriter.h>
 #include <Parser/CHColumnToSparkRow.h>
+#include <Parser/LocalExecutor.h>
+#include <Parser/ParserContext.h>
 #include <Parser/SerializedPlanParser.h>
 #include <Parser/SparkRowToCHColumn.h>
+#include <Parser/SubstraitParserUtils.h>
 #include <Parsers/ASTIdentifier.h>
 #include <Processors/Executors/PullingPipelineExecutor.h>
-#include <Processors/Formats/IOutputFormat.h>
 #include <Processors/QueryPlan/ExpressionStep.h>
 #include <Processors/QueryPlan/JoinStep.h>
-#include <Processors/QueryPlan/Optimizations/QueryPlanOptimizationSettings.h>
 #include <QueryPipeline/QueryPipelineBuilder.h>
 #include <Shuffle/ShuffleReader.h>
-#include <Shuffle/ShuffleSplitter.h>
-#include <Storages/CustomMergeTreeSink.h>
-#include <Storages/CustomStorageMergeTree.h>
 #include <Storages/MergeTree/MergeTreeData.h>
-#include <Storages/SelectQueryInfo.h>
-#include <Storages/SubstraitSource/ReadBufferBuilder.h>
+#include <Storages/MergeTree/SparkMergeTreeMeta.h>
+#include <Storages/MergeTree/SparkStorageMergeTree.h>
 #include <Storages/SubstraitSource/SubstraitFileSource.h>
 #include <benchmark/benchmark.h>
 #include <substrait/plan.pb.h>
-#include <Poco/Util/MapConfiguration.h>
 #include <Common/CHUtil.h>
 #include <Common/DebugUtils.h>
-#include <Common/MergeTreeTool.h>
 #include <Common/PODArray_fwd.h>
+#include <Common/QueryContext.h>
 #include <Common/Stopwatch.h>
 #include <Common/logger_useful.h>
 #include "testConfig.h"
@@ -60,6 +56,7 @@
 
 using namespace local_engine;
 using namespace dbms;
+using namespace DB;
 
 DB::ContextMutablePtr global_context;
 
@@ -92,7 +89,7 @@ DB::ContextMutablePtr global_context;
         substrait::ReadRel::LocalFiles::FileOrFiles::ParquetReadOptions parquet_format;
         file->mutable_parquet()->CopyFrom(parquet_format);
         auto builder = std::make_unique<QueryPipelineBuilder>();
-        builder->init(Pipe(std::make_shared<SubstraitFileSource>(SerializedPlanParser::global_context, header, files)));
+        builder->init(Pipe(std::make_shared<SubstraitFileSource>(QueryContext::globalContext(), header, files)));
 
         auto pipeline = QueryPipelineBuilder::getPipeline(std::move(*builder));
         auto executor = PullingPipelineExecutor(pipeline);
@@ -153,7 +150,8 @@ DB::ContextMutablePtr global_context;
                       "/home/kyligence/Documents/test-dataset/intel-gazelle-test-" + std::to_string(state.range(0)) + ".snappy.parquet",
                       std::move(schema))
                   .build();
-        local_engine::SerializedPlanParser parser(global_context);
+        auto parser_context = ParserContext::build(global_context, *plan);
+        local_engine::SerializedPlanParser parser(parser_context);
         auto local_executor = parser.createExecutor(*plan);
         state.ResumeTiming();
 
@@ -208,7 +206,8 @@ DB::ContextMutablePtr global_context;
                       "/home/kyligence/Documents/test-dataset/intel-gazelle-test-" + std::to_string(state.range(0)) + ".snappy.parquet",
                       std::move(schema))
                   .build();
-        local_engine::SerializedPlanParser parser(SerializedPlanParser::global_context);
+        auto parser_context = ParserContext::build(QueryContext::globalContext(), *plan);
+        local_engine::SerializedPlanParser parser(parser_context);
         auto local_executor = parser.createExecutor(*plan);
         state.ResumeTiming();
 
@@ -222,8 +221,7 @@ DB::ContextMutablePtr global_context;
 
 [[maybe_unused]] static void BM_MERGE_TREE_TPCH_Q6_FROM_TEXT(benchmark::State & state)
 {
-    SerializedPlanParser::global_context = global_context;
-    local_engine::SerializedPlanParser parser(SerializedPlanParser::global_context);
+    QueryContext::globalContext() = global_context;
     for (auto _ : state)
     {
         state.PauseTiming();
@@ -234,7 +232,10 @@ DB::ContextMutablePtr global_context;
         std::ifstream t(path);
         std::string str((std::istreambuf_iterator<char>(t)), std::istreambuf_iterator<char>());
         std::cout << "the plan from: " << path << std::endl;
-        auto local_executor = parser.createExecutor<false>(str);
+        auto plan = BinaryToMessage<substrait::Plan>(str);
+        auto parser_context = ParserContext::build(global_context, plan);
+        local_engine::SerializedPlanParser parser(parser_context);
+        auto local_executor = parser.createExecutor(plan);
         state.ResumeTiming();
         while (local_executor->hasNext()) [[maybe_unused]]
             auto * x = local_executor->nextColumnar();
@@ -272,7 +273,8 @@ DB::ContextMutablePtr global_context;
                       "/home/kyligence/Documents/test-dataset/intel-gazelle-test-" + std::to_string(state.range(0)) + ".snappy.parquet",
                       std::move(schema))
                   .build();
-        local_engine::SerializedPlanParser parser(SerializedPlanParser::global_context);
+        auto parser_context = ParserContext::build(QueryContext::globalContext(), *plan);
+        local_engine::SerializedPlanParser parser(parser_context);
 
         auto local_executor = parser.createExecutor(*plan);
         state.ResumeTiming();
@@ -308,7 +310,8 @@ DB::ContextMutablePtr global_context;
                       std::move(schema))
                   .build();
 
-        local_engine::SerializedPlanParser parser(SerializedPlanParser::global_context);
+        auto parser_context = ParserContext::build(QueryContext::globalContext(), *plan);
+        local_engine::SerializedPlanParser parser(parser_context);
         auto local_executor = parser.createExecutor(*plan);
         local_engine::SparkRowToCHColumn converter;
         while (local_executor->hasNext())
@@ -353,7 +356,8 @@ DB::ContextMutablePtr global_context;
                       "/home/kyligence/Documents/test-dataset/intel-gazelle-test-" + std::to_string(state.range(0)) + ".snappy.parquet",
                       std::move(schema))
                   .build();
-        local_engine::SerializedPlanParser parser(SerializedPlanParser::global_context);
+        auto parser_context = ParserContext::build(QueryContext::globalContext(), *plan);
+        local_engine::SerializedPlanParser parser(parser_context);
         auto local_executor = parser.createExecutor(*plan);
         local_engine::SparkRowToCHColumn converter;
         while (local_executor->hasNext())
@@ -585,8 +589,7 @@ DB::ContextMutablePtr global_context;
             readIntBinary(x, buf);
             readIntBinary(y, buf);
             readIntBinary(z, buf);
-            std::cout << std::to_string(x) + " " << std::to_string(y) + " " << std::to_string(z) + " "
-                      << "\n";
+            std::cout << std::to_string(x) + " " << std::to_string(y) + " " << std::to_string(z) + " " << "\n";
             data_buf.seek(x, SEEK_SET);
             assert(!data_buf.eof());
             std::string data;
@@ -790,7 +793,7 @@ public:
 
 struct MergeTreeWithSnapshot
 {
-    std::shared_ptr<local_engine::CustomStorageMergeTree> merge_tree;
+    std::shared_ptr<local_engine::SparkStorageMergeTree> merge_tree;
     std::shared_ptr<StorageSnapshot> snapshot;
     NamesAndTypesList columns;
 };
@@ -809,12 +812,12 @@ QueryPlanPtr readFromMergeTree(MergeTreeWithSnapshot storage)
 QueryPlanPtr joinPlan(QueryPlanPtr left, QueryPlanPtr right, String left_key, String right_key, size_t block_size = 8192)
 {
     auto join = std::make_shared<TableJoin>(
-        global_context->getSettings(), global_context->getGlobalTemporaryVolume(), global_context->getTempDataOnDisk());
-    auto left_columns = left->getCurrentDataStream().header.getColumnsWithTypeAndName();
-    auto right_columns = right->getCurrentDataStream().header.getColumnsWithTypeAndName();
+        global_context->getSettingsRef(), global_context->getGlobalTemporaryVolume(), global_context->getTempDataOnDisk());
+    auto left_columns = left->getCurrentHeader().getColumnsWithTypeAndName();
+    auto right_columns = right->getCurrentHeader().getColumnsWithTypeAndName();
     join->setKind(JoinKind::Left);
     join->setStrictness(JoinStrictness::All);
-    join->setColumnsFromJoinedTable(right->getCurrentDataStream().header.getNamesAndTypesList());
+    join->setColumnsFromJoinedTable(right->getCurrentHeader().getNamesAndTypesList());
     join->addDisjunct();
     ASTPtr lkey = std::make_shared<ASTIdentifier>(left_key);
     ASTPtr rkey = std::make_shared<ASTIdentifier>(right_key);
@@ -822,29 +825,29 @@ QueryPlanPtr joinPlan(QueryPlanPtr left, QueryPlanPtr right, String left_key, St
     for (const auto & column : join->columnsFromJoinedTable())
         join->addJoinedColumn(column);
 
-    auto left_keys = left->getCurrentDataStream().header.getNamesAndTypesList();
+    auto left_keys = left->getCurrentHeader().getNamesAndTypesList();
     join->addJoinedColumnsAndCorrectTypes(left_keys, true);
-    ActionsDAGPtr left_convert_actions = nullptr;
-    ActionsDAGPtr right_convert_actions = nullptr;
+    std::optional<ActionsDAG> left_convert_actions;
+    std::optional<ActionsDAG> right_convert_actions;
     std::tie(left_convert_actions, right_convert_actions) = join->createConvertingActions(left_columns, right_columns);
 
     if (right_convert_actions)
     {
-        auto converting_step = std::make_unique<ExpressionStep>(right->getCurrentDataStream(), right_convert_actions);
+        auto converting_step = std::make_unique<ExpressionStep>(right->getCurrentHeader(), std::move(*right_convert_actions));
         converting_step->setStepDescription("Convert joined columns");
         right->addStep(std::move(converting_step));
     }
 
     if (left_convert_actions)
     {
-        auto converting_step = std::make_unique<ExpressionStep>(right->getCurrentDataStream(), right_convert_actions);
+        auto converting_step = std::make_unique<ExpressionStep>(right->getCurrentHeader(), std::move(*right_convert_actions));
         converting_step->setStepDescription("Convert joined columns");
         left->addStep(std::move(converting_step));
     }
-    auto hash_join = std::make_shared<HashJoin>(join, right->getCurrentDataStream().header);
+    auto hash_join = std::make_shared<HashJoin>(join, right->getCurrentHeader());
 
     QueryPlanStepPtr join_step
-        = std::make_unique<JoinStep>(left->getCurrentDataStream(), right->getCurrentDataStream(), hash_join, block_size, 1, false);
+        = std::make_unique<JoinStep>(left->getCurrentHeader(), right->getCurrentHeader(), hash_join, block_size, 8192, 1,  NameSet{}, false, false);
 
     std::vector<QueryPlanPtr> plans;
     plans.emplace_back(std::move(left));
@@ -891,8 +894,8 @@ BENCHMARK(BM_ParquetRead)->Unit(benchmark::kMillisecond)->Iterations(10);
 
 int main(int argc, char ** argv)
 {
-    std::string empty;
-    BackendInitializerUtil::init(empty);
+    SparkConfigs::ConfigMap empty;
+    BackendInitializerUtil::initBackend(empty);
     SCOPE_EXIT({ BackendFinalizerUtil::finalizeGlobally(); });
 
     ::benchmark::Initialize(&argc, argv);
