@@ -21,8 +21,7 @@
 
 namespace gluten {
 
-Spill::Spill(Spill::SpillType type, uint32_t /* numPartitions */, const std::string& spillFile)
-    : type_(type), spillFile_(spillFile) {}
+Spill::Spill(Spill::SpillType type) : type_(type) {}
 
 Spill::~Spill() {
   if (is_) {
@@ -48,7 +47,7 @@ void Spill::insertPayload(
     Payload::Type payloadType,
     uint32_t numRows,
     const std::vector<bool>* isValidityBuffer,
-    uint64_t rawSize,
+    int64_t rawSize,
     arrow::MemoryPool* pool,
     arrow::util::Codec* codec) {
   // TODO: Add compression threshold.
@@ -81,4 +80,28 @@ void Spill::openForRead(uint64_t shuffleFileBufferSize) {
 Spill::SpillType Spill::type() const {
   return type_;
 }
+
+void Spill::setSpillFile(const std::string& spillFile) {
+  spillFile_ = spillFile;
+}
+
+void Spill::setSpillTime(int64_t spillTime) {
+  spillTime_ = spillTime;
+}
+
+void Spill::setCompressTime(int64_t compressTime) {
+  compressTime_ = compressTime;
+}
+
+std::string Spill::spillFile() const {
+  return spillFile_;
+}
+
+int64_t Spill::spillTime() const {
+  return spillTime_;
+}
+
+int64_t Spill::compressTime() const {
+  return compressTime_;
+}
 } // namespace gluten
