@@ -23,7 +23,7 @@
 #include <list>
 
 #include "shuffle/Payload.h"
-#include "utils/macros.h"
+#include "utils/Macros.h"
 
 namespace gluten {
 
@@ -31,7 +31,7 @@ class Spill final {
  public:
   enum SpillType { kSequentialSpill, kBatchedSpill };
 
-  Spill(SpillType type, uint32_t numPartitions, const std::string& spillFile);
+  Spill(SpillType type);
 
   ~Spill();
 
@@ -48,10 +48,22 @@ class Spill final {
       Payload::Type payloadType,
       uint32_t numRows,
       const std::vector<bool>* isValidityBuffer,
-      uint64_t rawSize,
+      int64_t rawSize,
       arrow::MemoryPool* pool,
       arrow::util::Codec* codec);
 
+  void setSpillFile(const std::string& spillFile);
+
+  void setSpillTime(int64_t spillTime);
+
+  void setCompressTime(int64_t compressTime);
+
+  std::string spillFile() const;
+
+  int64_t spillTime() const;
+
+  int64_t compressTime() const;
+
  private:
   struct PartitionPayload {
     uint32_t partitionId{};
@@ -62,6 +74,8 @@ class Spill final {
   std::shared_ptr<gluten::MmapFileStream> is_;
   std::list<PartitionPayload> partitionPayloads_{};
   std::string spillFile_;
+  int64_t spillTime_;
+  int64_t compressTime_;
 
   arrow::io::InputStream* rawIs_{nullptr};
 };
