@@ -22,9 +22,9 @@
 #include "memory/VeloxColumnarBatch.h"
 #include "substrait/SubstraitToVeloxPlan.h"
 #include "substrait/plan.pb.h"
-#include "utils/metrics.h"
+#include "utils/Metrics.h"
+#include "velox/common/config/Config.h"
 #include "velox/connectors/hive/iceberg/IcebergSplit.h"
-#include "velox/core/Config.h"
 #include "velox/core/PlanNode.h"
 #include "velox/exec/Task.h"
 
@@ -82,7 +82,7 @@ class WholeStageResultIterator : public ColumnarBatchIterator {
       std::vector<facebook::velox::core::PlanNodeId>& nodeIds);
 
   /// Create connector config.
-  std::shared_ptr<facebook::velox::Config> createConnectorConfig();
+  std::shared_ptr<facebook::velox::config::ConfigBase> createConnectorConfig();
 
   /// Construct partition columns.
   void constructPartitionColumns(
@@ -105,7 +105,7 @@ class WholeStageResultIterator : public ColumnarBatchIterator {
   VeloxMemoryManager* memoryManager_;
 
   /// Config, task and plan.
-  std::shared_ptr<Config> veloxCfg_;
+  std::shared_ptr<config::ConfigBase> veloxCfg_;
   const SparkTaskInfo taskInfo_;
   std::shared_ptr<facebook::velox::exec::Task> task_;
   std::shared_ptr<const facebook::velox::core::PlanNode> veloxPlan_;
@@ -120,7 +120,7 @@ class WholeStageResultIterator : public ColumnarBatchIterator {
   /// All the children plan node ids with postorder traversal.
   std::vector<facebook::velox::core::PlanNodeId> orderedNodeIds_;
 
-  /// Node ids should be ommited in metrics.
+  /// Node ids should be omitted in metrics.
   std::unordered_set<facebook::velox::core::PlanNodeId> omittedNodeIds_;
   std::vector<facebook::velox::core::PlanNodeId> scanNodeIds_;
   std::vector<std::shared_ptr<SplitInfo>> scanInfos_;
