@@ -21,8 +21,33 @@
 #include "memory/VeloxColumnarBatch.h"
 #include "velox/exec/Driver.h"
 #include "velox/exec/Operator.h"
+#include "velox/exec/Task.h"
+
+namespace {
+
+class SuspendedSection {
+ public:
+  explicit SuspendedSection(facebook::velox::exec::Driver* driver) : driver_(driver) {
+    if (driver_->task()->enterSuspended(driver->state()) != facebook::velox::exec::StopReason::kNone) {
+      VELOX_FAIL("Terminate detected when entering suspended section");
+    }
+  }
+
+  virtual ~SuspendedSection() {
+    if (driver_->task()->leaveSuspended(driver_->state()) != facebook::velox::exec::StopReason::kNone) {
+      LOG(WARNING) << "Terminate detected when leaving suspended section for driver " << driver_->driverCtx()->driverId
+                   << " from task " << driver_->task()->taskId();
+    }
+  }
+
+ private:
+  facebook::velox::exec::Driver* const driver_;
+};
+
+} // namespace
 
 namespace gluten {
+
 class RowVectorStream {
  public:
   explicit RowVectorStream(
@@ -45,8 +70,8 @@ class RowVectorStream {
       // possibility that this spill call hangs. See https://github.com/apache/incubator-gluten/issues/7243.
       // As of now, non-zero running threads usually happens when:
       // 1. Task A spills task B;
-      // 2. Task A trys to grow buffers created by task B, during which spill is requested on task A again.
-      facebook::velox::exec::SuspendedSection ss(driverCtx_->driver);
+      // 2. Task A tries to grow buffers created by task B, during which spill is requested on task A again.
+      SuspendedSection ss(driverCtx_->driver);
       hasNext = iterator_->hasNext();
     }
     if (!hasNext) {
@@ -55,7 +80,7 @@ class RowVectorStream {
     return hasNext;
   }
 
-  // Convert arrow batch to rowvector and use new output columns
+  // Convert arrow batch to row vector and use new output columns
   facebook::velox::RowVectorPtr next() {
     if (finished_) {
       return nullptr;
@@ -64,7 +89,7 @@ class RowVectorStream {
     {
       // We are leaving Velox task execution and are probably entering Spark code through JNI. Suspend the current
       // driver to make the current task open to spilling.
-      facebook::velox::exec::SuspendedSection ss(driverCtx_->driver);
+      SuspendedSection ss(driverCtx_->driver);
       cb = iterator_->next();
     }
     const std::shared_ptr<VeloxColumnarBatch>& vb = VeloxColumnarBatch::from(pool_, cb);
