@@ -16,6 +16,7 @@
  */
 
 #include "VeloxShuffleReader.h"
+#include "GlutenByteStream.h"
 
 #include <arrow/array/array_binary.h>
 #include <arrow/io/buffered.h>
@@ -25,17 +26,18 @@
 #include "shuffle/Utils.h"
 #include "utils/Common.h"
 #include "utils/Compression.h"
+#include "utils/Macros.h"
 #include "utils/Timer.h"
 #include "utils/VeloxArrowUtils.h"
-#include "utils/macros.h"
+#include "velox/row/CompactRow.h"
 #include "velox/serializers/PrestoSerializer.h"
 #include "velox/vector/ComplexVector.h"
 #include "velox/vector/FlatVector.h"
 #include "velox/vector/arrow/Bridge.h"
 
+#include <algorithm>
 #include <iostream>
 
-// using namespace facebook;
 using namespace facebook::velox;
 
 namespace gluten {
@@ -175,7 +177,7 @@ VectorPtr readFlatVector<TypeKind::VARBINARY>(
 std::unique_ptr<ByteInputStream> toByteStream(uint8_t* data, int32_t size) {
   std::vector<ByteRange> byteRanges;
   byteRanges.push_back(ByteRange{data, size, 0});
-  auto byteStream = std::make_unique<ByteInputStream>(byteRanges);
+  auto byteStream = std::make_unique<BufferInputStream>(byteRanges);
   return byteStream;
 }
 
@@ -282,23 +284,20 @@ std::shared_ptr<VeloxColumnarBatch> makeColumnarBatch(
 
 } // namespace
 
-VeloxShuffleReader::VeloxShuffleReader(std::unique_ptr<DeserializerFactory> factory)
-    : ShuffleReader(std::move(factory)) {}
-
-VeloxColumnarBatchDeserializer::VeloxColumnarBatchDeserializer(
+VeloxHashShuffleReaderDeserializer::VeloxHashShuffleReaderDeserializer(
     std::shared_ptr<arrow::io::InputStream> in,
     const std::shared_ptr<arrow::Schema>& schema,
     const std::shared_ptr<arrow::util::Codec>& codec,
     const facebook::velox::RowTypePtr& rowType,
     int32_t batchSize,
+    int64_t bufferSize,
     arrow::MemoryPool* memoryPool,
     facebook::velox::memory::MemoryPool* veloxPool,
     std::vector<bool>* isValidityBuffer,
     bool hasComplexType,
     int64_t& deserializeTime,
     int64_t& decompressTime)
-    : in_(std::move(in)),
-      schema_(schema),
+    : schema_(schema),
       codec_(codec),
       rowType_(rowType),
       batchSize_(batchSize),
@@ -307,15 +306,17 @@ VeloxColumnarBatchDeserializer::VeloxColumnarBatchDeserializer(
       isValidityBuffer_(isValidityBuffer),
       hasComplexType_(hasComplexType),
       deserializeTime_(deserializeTime),
-      decompressTime_(decompressTime) {}
+      decompressTime_(decompressTime) {
+  GLUTEN_ASSIGN_OR_THROW(in_, arrow::io::BufferedInputStream::Create(bufferSize, memoryPool, std::move(in)));
+}
 
-std::shared_ptr<ColumnarBatch> VeloxColumnarBatchDeserializer::next() {
+std::shared_ptr<ColumnarBatch> VeloxHashShuffleReaderDeserializer::next() {
   if (hasComplexType_) {
-    uint32_t numRows;
+    uint32_t numRows = 0;
     GLUTEN_ASSIGN_OR_THROW(
         auto arrowBuffers,
-        BlockPayload::deserialize(in_.get(), schema_, codec_, memoryPool_, numRows, decompressTime_));
-    if (numRows == 0) {
+        BlockPayload::deserialize(in_.get(), codec_, memoryPool_, numRows, deserializeTime_, decompressTime_));
+    if (arrowBuffers.empty()) {
       // Reach EOS.
       return nullptr;
     }
@@ -333,8 +334,9 @@ std::shared_ptr<ColumnarBatch> VeloxColumnarBatchDeserializer::next() {
   uint32_t numRows = 0;
   while (!merged_ || merged_->numRows() < batchSize_) {
     GLUTEN_ASSIGN_OR_THROW(
-        arrowBuffers, BlockPayload::deserialize(in_.get(), schema_, codec_, memoryPool_, numRows, decompressTime_));
-    if (numRows == 0) {
+        arrowBuffers,
+        BlockPayload::deserialize(in_.get(), codec_, memoryPool_, numRows, deserializeTime_, decompressTime_));
+    if (arrowBuffers.empty()) {
       reachEos_ = true;
       break;
     }
@@ -367,47 +369,172 @@ std::shared_ptr<ColumnarBatch> VeloxColumnarBatchDeserializer::next() {
   return columnarBatch;
 }
 
-VeloxColumnarBatchDeserializerFactory::VeloxColumnarBatchDeserializerFactory(
+VeloxSortShuffleReaderDeserializer::VeloxSortShuffleReaderDeserializer(
+    std::shared_ptr<arrow::io::InputStream> in,
     const std::shared_ptr<arrow::Schema>& schema,
     const std::shared_ptr<arrow::util::Codec>& codec,
-    const facebook::velox::common::CompressionKind veloxCompressionType,
     const RowTypePtr& rowType,
     int32_t batchSize,
+    int64_t bufferSize,
     arrow::MemoryPool* memoryPool,
-    std::shared_ptr<facebook::velox::memory::MemoryPool> veloxPool,
-    ShuffleWriterType shuffleWriterType)
+    facebook::velox::memory::MemoryPool* veloxPool,
+    int64_t& deserializeTime,
+    int64_t& decompressTime)
     : schema_(schema),
       codec_(codec),
-      veloxCompressionType_(veloxCompressionType),
       rowType_(rowType),
       batchSize_(batchSize),
-      memoryPool_(memoryPool),
+      arrowPool_(memoryPool),
       veloxPool_(veloxPool),
-      shuffleWriterType_(shuffleWriterType) {
-  initFromSchema();
+      deserializeTime_(deserializeTime),
+      decompressTime_(decompressTime) {
+  GLUTEN_ASSIGN_OR_THROW(in_, arrow::io::BufferedInputStream::Create(bufferSize, memoryPool, std::move(in)));
 }
 
-std::unique_ptr<ColumnarBatchIterator> VeloxColumnarBatchDeserializerFactory::createDeserializer(
-    std::shared_ptr<arrow::io::InputStream> in) {
-  if (shuffleWriterType_ == kHashShuffle) {
-    return std::make_unique<VeloxColumnarBatchDeserializer>(
-        std::move(in),
-        schema_,
-        codec_,
-        rowType_,
-        batchSize_,
-        memoryPool_,
-        veloxPool_.get(),
-        &isValidityBuffer_,
-        hasComplexType_,
-        deserializeTime_,
-        decompressTime_);
+std::shared_ptr<ColumnarBatch> VeloxSortShuffleReaderDeserializer::next() {
+  if (reachedEos_) {
+    if (cachedRows_ > 0) {
+      return deserializeToBatch();
+    }
+    return nullptr;
   }
-  return std::make_unique<VeloxShuffleReaderOutStreamWrapper>(
-      veloxPool_, rowType_, batchSize_, veloxCompressionType_, deserializeTime_, std::move(in));
+
+  if (cachedRows_ >= batchSize_) {
+    return deserializeToBatch();
+  }
+
+  while (cachedRows_ < batchSize_) {
+    uint32_t numRows = 0;
+    GLUTEN_ASSIGN_OR_THROW(
+        auto arrowBuffers,
+        BlockPayload::deserialize(in_.get(), codec_, arrowPool_, numRows, deserializeTime_, decompressTime_));
+
+    if (arrowBuffers.empty()) {
+      reachedEos_ = true;
+      if (cachedRows_ > 0) {
+        return deserializeToBatch();
+      }
+      return nullptr;
+    }
+
+    if (numRows > 0) {
+      auto buffer = std::move(arrowBuffers[0]);
+      cachedInputs_.emplace_back(numRows, wrapInBufferViewAsOwner(buffer->data(), buffer->size(), buffer));
+      cachedRows_ += numRows;
+    } else {
+      // numRows = 0 indicates that we read a segment of a large row.
+      readLargeRow(arrowBuffers);
+    }
+  }
+  return deserializeToBatch();
 }
 
-VeloxShuffleReaderOutStreamWrapper::VeloxShuffleReaderOutStreamWrapper(
+std::shared_ptr<ColumnarBatch> VeloxSortShuffleReaderDeserializer::deserializeToBatch() {
+  ScopedTimer timer(&deserializeTime_);
+  std::vector<std::string_view> data;
+  data.reserve(std::min(cachedRows_, batchSize_));
+
+  uint32_t readRows = 0;
+  auto cur = cachedInputs_.begin();
+  while (readRows < batchSize_ && cur != cachedInputs_.end()) {
+    auto buffer = cur->second;
+    const auto* rawBuffer = buffer->as<char>();
+    while (rowOffset_ < cur->first && readRows < batchSize_) {
+      auto rowSize = *(RowSizeType*)(rawBuffer + byteOffset_) - sizeof(RowSizeType);
+      byteOffset_ += sizeof(RowSizeType);
+      data.push_back(std::string_view(rawBuffer + byteOffset_, rowSize));
+      byteOffset_ += rowSize;
+      ++rowOffset_;
+      ++readRows;
+    }
+    if (rowOffset_ == cur->first) {
+      rowOffset_ = 0;
+      byteOffset_ = 0;
+      ++cur;
+    }
+  }
+  cachedRows_ -= readRows;
+  auto rowVector = facebook::velox::row::CompactRow::deserialize(data, rowType_, veloxPool_);
+  // Free memory.
+  auto iter = cachedInputs_.begin();
+  while (iter++ != cur) {
+    cachedInputs_.pop_front();
+  }
+  return std::make_shared<VeloxColumnarBatch>(std::move(rowVector));
+}
+
+void VeloxSortShuffleReaderDeserializer::readLargeRow(std::vector<std::shared_ptr<arrow::Buffer>>& arrowBuffers) {
+  // Cache the read segment.
+  std::vector<std::shared_ptr<arrow::Buffer>> buffers;
+  auto rowSize = *reinterpret_cast<RowSizeType*>(const_cast<uint8_t*>(arrowBuffers[0]->data()));
+  RowSizeType bufferSize = arrowBuffers[0]->size();
+  buffers.emplace_back(std::move(arrowBuffers[0]));
+  // Read and cache the remaining segments.
+  uint32_t numRows;
+  while (bufferSize < rowSize) {
+    GLUTEN_ASSIGN_OR_THROW(
+        arrowBuffers,
+        BlockPayload::deserialize(in_.get(), codec_, arrowPool_, numRows, deserializeTime_, decompressTime_));
+    VELOX_DCHECK_EQ(numRows, 0);
+    bufferSize += arrowBuffers[0]->size();
+    buffers.emplace_back(std::move(arrowBuffers[0]));
+  }
+  VELOX_CHECK_EQ(bufferSize, rowSize);
+  // Merge all segments.
+  GLUTEN_ASSIGN_OR_THROW(std::shared_ptr<arrow::Buffer> rowBuffer, arrow::AllocateBuffer(rowSize, arrowPool_));
+  RowSizeType bytes = 0;
+  auto* dst = rowBuffer->mutable_data();
+  for (const auto& buffer : buffers) {
+    VELOX_DCHECK_NOT_NULL(buffer);
+    gluten::fastCopy(dst + bytes, buffer->data(), buffer->size());
+    bytes += buffer->size();
+  }
+  cachedInputs_.emplace_back(1, wrapInBufferViewAsOwner(rowBuffer->data(), rowSize, rowBuffer));
+  cachedRows_++;
+}
+
+class VeloxRssSortShuffleReaderDeserializer::VeloxInputStream : public facebook::velox::GlutenByteInputStream {
+ public:
+  VeloxInputStream(std::shared_ptr<arrow::io::InputStream> input, facebook::velox::BufferPtr buffer);
+
+  bool hasNext();
+
+  void next(bool throwIfPastEnd) override;
+
+  std::shared_ptr<arrow::io::InputStream> in_;
+  const facebook::velox::BufferPtr buffer_;
+  uint64_t offset_ = -1;
+};
+
+VeloxRssSortShuffleReaderDeserializer::VeloxInputStream::VeloxInputStream(
+    std::shared_ptr<arrow::io::InputStream> input,
+    facebook::velox::BufferPtr buffer)
+    : in_(std::move(input)), buffer_(std::move(buffer)) {
+  next(true);
+}
+
+bool VeloxRssSortShuffleReaderDeserializer::VeloxInputStream::hasNext() {
+  if (offset_ == 0) {
+    return false;
+  }
+  if (ranges()[0].position >= ranges()[0].size) {
+    next(true);
+    return offset_ != 0;
+  }
+  return true;
+}
+
+void VeloxRssSortShuffleReaderDeserializer::VeloxInputStream::next(bool throwIfPastEnd) {
+  const uint32_t readBytes = buffer_->capacity();
+  offset_ = in_->Read(readBytes, buffer_->asMutable<char>()).ValueOr(0);
+  if (offset_ > 0) {
+    int32_t realBytes = offset_;
+    VELOX_CHECK_LT(0, realBytes, "Reading past end of file.");
+    setRange({buffer_->asMutable<uint8_t>(), realBytes, 0});
+  }
+}
+
+VeloxRssSortShuffleReaderDeserializer::VeloxRssSortShuffleReaderDeserializer(
     const std::shared_ptr<facebook::velox::memory::MemoryPool>& veloxPool,
     const RowTypePtr& rowType,
     int32_t batchSize,
@@ -418,6 +545,7 @@ VeloxShuffleReaderOutStreamWrapper::VeloxShuffleReaderOutStreamWrapper(
       rowType_(rowType),
       batchSize_(batchSize),
       veloxCompressionType_(veloxCompressionType),
+      serde_(getNamedVectorSerde(facebook::velox::VectorSerde::Kind::kPresto)),
       deserializeTime_(deserializeTime) {
   constexpr uint64_t kMaxReadBufferSize = (1 << 20) - AlignedBuffer::kPaddedSize;
   auto buffer = AlignedBuffer::allocate<char>(kMaxReadBufferSize, veloxPool_.get());
@@ -425,7 +553,7 @@ VeloxShuffleReaderOutStreamWrapper::VeloxShuffleReaderOutStreamWrapper(
   serdeOptions_ = {false, veloxCompressionType_};
 }
 
-std::shared_ptr<ColumnarBatch> VeloxShuffleReaderOutStreamWrapper::next() {
+std::shared_ptr<ColumnarBatch> VeloxRssSortShuffleReaderDeserializer::next() {
   if (!in_->hasNext()) {
     return nullptr;
   }
@@ -433,7 +561,7 @@ std::shared_ptr<ColumnarBatch> VeloxShuffleReaderOutStreamWrapper::next() {
   ScopedTimer timer(&deserializeTime_);
 
   RowVectorPtr rowVector;
-  VectorStreamGroup::read(in_.get(), veloxPool_.get(), rowType_, &rowVector, &serdeOptions_);
+  VectorStreamGroup::read(in_.get(), veloxPool_.get(), rowType_, serde_, &rowVector, &serdeOptions_);
 
   if (rowVector->size() >= batchSize_) {
     return std::make_shared<VeloxColumnarBatch>(std::move(rowVector));
@@ -441,13 +569,72 @@ std::shared_ptr<ColumnarBatch> VeloxShuffleReaderOutStreamWrapper::next() {
 
   while (rowVector->size() < batchSize_ && in_->hasNext()) {
     RowVectorPtr rowVectorTemp;
-    VectorStreamGroup::read(in_.get(), veloxPool_.get(), rowType_, &rowVectorTemp, &serdeOptions_);
+    VectorStreamGroup::read(in_.get(), veloxPool_.get(), rowType_, serde_, &rowVectorTemp, &serdeOptions_);
     rowVector->append(rowVectorTemp.get());
   }
 
   return std::make_shared<VeloxColumnarBatch>(std::move(rowVector));
 }
 
+VeloxColumnarBatchDeserializerFactory::VeloxColumnarBatchDeserializerFactory(
+    const std::shared_ptr<arrow::Schema>& schema,
+    const std::shared_ptr<arrow::util::Codec>& codec,
+    const facebook::velox::common::CompressionKind veloxCompressionType,
+    const RowTypePtr& rowType,
+    int32_t batchSize,
+    int64_t bufferSize,
+    arrow::MemoryPool* memoryPool,
+    std::shared_ptr<facebook::velox::memory::MemoryPool> veloxPool,
+    ShuffleWriterType shuffleWriterType)
+    : schema_(schema),
+      codec_(codec),
+      veloxCompressionType_(veloxCompressionType),
+      rowType_(rowType),
+      batchSize_(batchSize),
+      bufferSize_(bufferSize),
+      memoryPool_(memoryPool),
+      veloxPool_(veloxPool),
+      shuffleWriterType_(shuffleWriterType) {
+  initFromSchema();
+}
+
+std::unique_ptr<ColumnarBatchIterator> VeloxColumnarBatchDeserializerFactory::createDeserializer(
+    std::shared_ptr<arrow::io::InputStream> in) {
+  switch (shuffleWriterType_) {
+    case ShuffleWriterType::kHashShuffle:
+      return std::make_unique<VeloxHashShuffleReaderDeserializer>(
+          std::move(in),
+          schema_,
+          codec_,
+          rowType_,
+          batchSize_,
+          bufferSize_,
+          memoryPool_,
+          veloxPool_.get(),
+          &isValidityBuffer_,
+          hasComplexType_,
+          deserializeTime_,
+          decompressTime_);
+    case ShuffleWriterType::kSortShuffle:
+      return std::make_unique<VeloxSortShuffleReaderDeserializer>(
+          std::move(in),
+          schema_,
+          codec_,
+          rowType_,
+          batchSize_,
+          bufferSize_,
+          memoryPool_,
+          veloxPool_.get(),
+          deserializeTime_,
+          decompressTime_);
+    case ShuffleWriterType::kRssSortShuffle:
+      return std::make_unique<VeloxRssSortShuffleReaderDeserializer>(
+          veloxPool_, rowType_, batchSize_, veloxCompressionType_, deserializeTime_, std::move(in));
+    default:
+      throw gluten::GlutenException("Unsupported shuffle writer type: " + std::to_string(shuffleWriterType_));
+  }
+}
+
 arrow::MemoryPool* VeloxColumnarBatchDeserializerFactory::getPool() {
   return memoryPool_;
 }
@@ -494,29 +681,7 @@ void VeloxColumnarBatchDeserializerFactory::initFromSchema() {
   }
 }
 
-VeloxInputStream::VeloxInputStream(std::shared_ptr<arrow::io::InputStream> input, facebook::velox::BufferPtr buffer)
-    : in_(std::move(input)), buffer_(std::move(buffer)) {
-  next(true);
-}
+VeloxShuffleReader::VeloxShuffleReader(std::unique_ptr<DeserializerFactory> factory)
+    : ShuffleReader(std::move(factory)) {}
 
-bool VeloxInputStream::hasNext() {
-  if (offset_ == 0) {
-    return false;
-  }
-  if (ranges()[0].position >= ranges()[0].size) {
-    next(true);
-    return offset_ != 0;
-  }
-  return true;
-}
-
-void VeloxInputStream::next(bool throwIfPastEnd) {
-  const uint32_t readBytes = buffer_->capacity();
-  offset_ = in_->Read(readBytes, buffer_->asMutable<char>()).ValueOr(0);
-  if (offset_ > 0) {
-    int32_t realBytes = offset_;
-    VELOX_CHECK_LT(0, realBytes, "Reading past end of file.");
-    setRange({buffer_->asMutable<uint8_t>(), realBytes, 0});
-  }
-}
 } // namespace gluten
