@@ -66,14 +66,17 @@ struct ShuffleTestParams {
   ShuffleWriterType shuffleWriterType;
   PartitionWriterType partitionWriterType;
   arrow::Compression::type compressionType;
-  int32_t compressionThreshold;
-  int32_t mergeBufferSize;
+  int32_t compressionThreshold{0};
+  int32_t mergeBufferSize{0};
+  int32_t compressionBufferSize{0};
+  bool useRadixSort{false};
 
   std::string toString() const {
     std::ostringstream out;
     out << "shuffleWriterType = " << shuffleWriterType << ", partitionWriterType = " << partitionWriterType
         << ", compressionType = " << compressionType << ", compressionThreshold = " << compressionThreshold
-        << ", mergeBufferSize = " << mergeBufferSize;
+        << ", mergeBufferSize = " << mergeBufferSize << ", compressionBufferSize = " << compressionBufferSize
+        << ", useRadixSort = " << (useRadixSort ? "true" : "false");
     return out.str();
   }
 };
@@ -87,6 +90,10 @@ class VeloxShuffleWriterTestBase : public facebook::velox::test::VectorTestBase
 
  protected:
   void setUp() {
+    if (!isRegisteredNamedVectorSerde(facebook::velox::VectorSerde::Kind::kPresto)) {
+      // RSS shuffle serde.
+      facebook::velox::serializer::presto::PrestoVectorSerde::registerNamedVectorSerde();
+    }
     // Set up test data.
     children1_ = {
         makeNullableFlatVector<int8_t>({1, 2, 3, std::nullopt, 4, std::nullopt, 5, 6, std::nullopt, 7}),
@@ -195,9 +202,12 @@ class VeloxShuffleWriterTestBase : public facebook::velox::test::VectorTestBase
     inputVectorComplex_ = makeRowVector(childrenComplex_);
   }
 
-  arrow::Status splitRowVector(VeloxShuffleWriter& shuffleWriter, facebook::velox::RowVectorPtr vector) {
+  arrow::Status splitRowVector(
+      VeloxShuffleWriter& shuffleWriter,
+      facebook::velox::RowVectorPtr vector,
+      int64_t memLimit = ShuffleWriter::kMinMemLimit) {
     std::shared_ptr<ColumnarBatch> cb = std::make_shared<VeloxColumnarBatch>(vector);
-    return shuffleWriter.write(cb, ShuffleWriter::kMinMemLimit);
+    return shuffleWriter.write(cb, memLimit);
   }
 
   // Create multiple local dirs and join with comma.
@@ -250,6 +260,8 @@ class VeloxShuffleWriterTest : public ::testing::TestWithParam<ShuffleTestParams
     RETURN_NOT_OK(VeloxShuffleWriterTestBase::initShuffleWriterOptions());
 
     ShuffleTestParams params = GetParam();
+    shuffleWriterOptions_.useRadixSort = params.useRadixSort;
+    shuffleWriterOptions_.sortEvictBufferSize = params.compressionBufferSize;
     partitionWriterOptions_.compressionType = params.compressionType;
     switch (partitionWriterOptions_.compressionType) {
       case arrow::Compression::UNCOMPRESSED:
@@ -275,19 +287,18 @@ class VeloxShuffleWriterTest : public ::testing::TestWithParam<ShuffleTestParams
       ShuffleWriterOptions shuffleWriterOptions,
       uint32_t numPartitions,
       int32_t bufferSize) {
-    std::shared_ptr<VeloxShuffleWriter> shuffleWriter;
-    if (GetParam().shuffleWriterType == kHashShuffle) {
+    if (shuffleWriterOptions.shuffleWriterType == ShuffleWriterType::kHashShuffle) {
       shuffleWriterOptions.bufferSize = bufferSize;
-      GLUTEN_ASSIGN_OR_THROW(
-          shuffleWriter,
-          VeloxHashBasedShuffleWriter::create(
-              numPartitions, std::move(partitionWriter), std::move(shuffleWriterOptions), pool_, arrowPool));
-    } else if (GetParam().shuffleWriterType == kSortShuffle) {
-      GLUTEN_ASSIGN_OR_THROW(
-          shuffleWriter,
-          VeloxSortBasedShuffleWriter::create(
-              numPartitions, std::move(partitionWriter), std::move(shuffleWriterOptions), pool_, arrowPool));
     }
+    GLUTEN_ASSIGN_OR_THROW(
+        auto shuffleWriter,
+        VeloxShuffleWriter::create(
+            GetParam().shuffleWriterType,
+            numPartitions,
+            std::move(partitionWriter),
+            std::move(shuffleWriterOptions),
+            pool_,
+            arrowPool));
     return shuffleWriter;
   }
 
@@ -355,6 +366,7 @@ class VeloxShuffleWriterTest : public ::testing::TestWithParam<ShuffleTestParams
         veloxCompressionType,
         rowType,
         std::numeric_limits<int32_t>::max(),
+        kDefaultReadBufferSize,
         defaultArrowMemoryPool().get(),
         pool_,
         GetParam().shuffleWriterType);
@@ -485,13 +497,13 @@ class RangePartitioningShuffleWriter : public MultiplePartitioningShuffleWriter
 
     auto pid1 = makeRowVector({makeFlatVector<int32_t>({0, 1, 0, 1, 0, 1, 0, 1, 0, 1})});
     auto rangeVector1 = makeRowVector(inputVector1_->children());
-    compositeBatch1_ = CompositeColumnarBatch::create(
-        {std::make_shared<VeloxColumnarBatch>(pid1), std::make_shared<VeloxColumnarBatch>(rangeVector1)});
+    compositeBatch1_ = VeloxColumnarBatch::compose(
+        pool(), {std::make_shared<VeloxColumnarBatch>(pid1), std::make_shared<VeloxColumnarBatch>(rangeVector1)});
 
     auto pid2 = makeRowVector({makeFlatVector<int32_t>({0, 1})});
     auto rangeVector2 = makeRowVector(inputVector2_->children());
-    compositeBatch2_ = CompositeColumnarBatch::create(
-        {std::make_shared<VeloxColumnarBatch>(pid2), std::make_shared<VeloxColumnarBatch>(rangeVector2)});
+    compositeBatch2_ = VeloxColumnarBatch::compose(
+        pool(), {std::make_shared<VeloxColumnarBatch>(pid2), std::make_shared<VeloxColumnarBatch>(rangeVector2)});
   }
 
   std::shared_ptr<VeloxShuffleWriter> createShuffleWriter(arrow::MemoryPool* arrowPool) override {
@@ -532,7 +544,7 @@ class RoundRobinPartitioningShuffleWriter : public MultiplePartitioningShuffleWr
   }
 };
 
-class VeloxShuffleWriterMemoryTest : public VeloxShuffleWriterTestBase, public testing::Test {
+class VeloxHashShuffleWriterMemoryTest : public VeloxShuffleWriterTestBase, public testing::Test {
  protected:
   static void SetUpTestCase() {
     facebook::velox::memory::MemoryManager::testingSetInstance({});
@@ -546,7 +558,7 @@ class VeloxShuffleWriterMemoryTest : public VeloxShuffleWriterTestBase, public t
         PartitionWriterType::kLocal, numPartitions, dataFile_, localDirs_, partitionWriterOptions_, arrowPool);
     GLUTEN_ASSIGN_OR_THROW(
         auto shuffleWriter,
-        VeloxHashBasedShuffleWriter::create(
+        VeloxHashShuffleWriter::create(
             numPartitions, std::move(partitionWriter), std::move(shuffleWriterOptions_), pool_, arrowPool));
     return shuffleWriter;
   }
