@@ -30,11 +30,6 @@ import org.apache.spark.sql.execution.SparkPlan
  * TODO: Ideally for such API we'd better to allow multiple alternative outputs.
  */
 trait RewriteSingleNode {
+  def isRewritable(plan: SparkPlan): Boolean
   def rewrite(plan: SparkPlan): SparkPlan
 }
-
-object RewriteSingleNode {
-  def allRules(): Seq[RewriteSingleNode] = {
-    Seq(RewriteIn, RewriteMultiChildrenCount, RewriteJoin, PullOutPreProject, PullOutPostProject)
-  }
-}
