@@ -16,7 +16,7 @@
  */
 package org.apache.spark.sql
 
-import org.apache.gluten.GlutenConfig
+import org.apache.gluten.config.GlutenConfig
 import org.apache.gluten.execution.{BatchScanExecTransformer, FileSourceScanExecTransformer, FilterExecTransformerBase}
 
 import org.apache.spark.SparkConf
@@ -776,6 +776,49 @@ class GlutenDynamicPartitionPruningV1SuiteAEOn
       checkAnswer(df, Row(1000, 1) :: Row(1010, 2) :: Row(1020, 2) :: Nil)
     }
   }
+
+  testGluten("Filter with DynamicPruningExpression") {
+    withTable("fact_stats_non_partition") {
+      spark
+        .table("fact_stats")
+        .write
+        .format(tableFormat)
+        .saveAsTable("fact_stats_non_partition")
+
+      withSQLConf(
+        SQLConf.DYNAMIC_PARTITION_PRUNING_REUSE_BROADCAST_ONLY.key -> "true",
+        SQLConf.EXCHANGE_REUSE_ENABLED.key -> "false") {
+        val df = sql("""
+                       |SELECT f.date_id, f.product_id, f.units_sold, f.store_id FROM (
+                       | select * from fact_stats
+                       | union all
+                       | select * from fact_stats_non_partition
+                       |) f
+                       |JOIN dim_stats s
+                       |ON f.store_id = s.store_id WHERE s.country = 'DE'
+        """.stripMargin)
+        checkAnswer(
+          df,
+          Row(1030, 2, 10, 3) ::
+            Row(1040, 2, 50, 3) ::
+            Row(1050, 2, 50, 3) ::
+            Row(1060, 2, 50, 3) ::
+            Row(1030, 2, 10, 3) ::
+            Row(1040, 2, 50, 3) ::
+            Row(1050, 2, 50, 3) ::
+            Row(1060, 2, 50, 3) :: Nil
+        )
+        val filters = collect(df.queryExecution.executedPlan) { case f: FilterExec => f }
+        assert(filters.isEmpty)
+        val filterTransformerWithDPPs = collect(df.queryExecution.executedPlan) {
+          case f: FilterExecTransformerBase
+              if f.cond.exists(_.isInstanceOf[DynamicPruningExpression]) =>
+            f
+        }
+        assert(filterTransformerWithDPPs.nonEmpty)
+      }
+    }
+  }
 }
 
 abstract class GlutenDynamicPartitionPruningV2Suite extends GlutenDynamicPartitionPruningSuiteBase {
